/******************************************************************************
 * xrun - a quick and easy way to run programs under X without having to drop
 * to a terminal window.
 Copyright (C) 1998  by Christopher Lansdown

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/

#include "main.h"

/*****************************************************************************/
/* Begin Global Variables */

extern Config config;

/* End Global Variables */
/*****************************************************************************/ 

int config_init()
{
  char *home_dir, *home_env;
  FILE *stream;
  char line[1025], *chr;
  int state = 0; /* 0 = general config; 1 = history */
  int i = 1;
  int j = 0;
  int linelen = 0;

  /* Clear all flags */
  config.flags &= 0;
  /* Set the numbers to zero. */
  config.bash_number = 
    config.config_number = 
    config.bash_length = 
    config.config_length = 0;

  home_env = getenv("HOME");
  home_dir = calloc((strlen(home_env) + 2 + strlen(HOME_FILE)), 1);
  strcpy(home_dir, home_env);
  strcat(home_dir, "/");
  strcat(home_dir, HOME_FILE);

  config.config = calloc(strlen(home_dir) + 1, 1);
  strcpy(config.config, home_dir);
  free(home_dir);

  stream = fopen(config.config, "r");
  
  /* Set up the default values */
  config.flags = 0;
  config.history=NULL;
  config.config_length = 0;
  config.bash_length = 0;
  config.config_number = 0;
  config.bash_number = 0;
  config.x = -1;
  config.y = -1;
  config.x_pos = -2;
  config.y_pos = -2;

  if(stream == NULL)
    return TRUE;

  while(fgets(line, 1024, stream)){
    if(state == 0) {
      /* Get wrid of comments */
      if(is_comment(line))
	continue;
      strip_comment(line);

      /* Convert the line to upper  case so that matches aren't case sensitive (if not in history)*/
      linelen = strlen(line);
      if(state == 0)
	for(j = 0; j < linelen; j++)
	  line[j] = toupper(line[j]);
      
      /* Start processing the config file directives. */
      if(strstr(line, "USE")) {
	if(strstr(line, "BASH_HISTORY"))
	  config.flags |= CONFIG_BASH_HISTORY;
	if(strstr(line, "CONFIG_HISTORY"))
	  config.flags |= CONFIG_CONFIG_HISTORY;
      }

      if(strstr(line, "ORDER"))
	if(strstr(line, "BASH") < strstr(line, "CONFIG"))
	  config.flags |= CONFIG_BASH_FIRST;
	else
	  config.flags &= ~CONFIG_BASH_FIRST;
      
      if(strstr(line, "HISTORY_EDITABLE"))
	if(strstr(line, "YES"))
	  config.flags |= CONFIG_HISTORY_EDITABLE;
	else
	  config.flags &= ~CONFIG_HISTORY_EDITABLE;
      
      if(strstr(line, "CONFIG_HISTORY_LENGTH"))
	if(sscanf(line, "CONFIG_HISTORY_LENGTH %i", &config.config_length) < 1)
	  config.config_length = 15;
      
      if(strstr(line, "BASH_HISTORY_LENGTH"))
	if(sscanf(line, "BASH_HISTORY_LENGTH %i", &config.bash_length) < 1)
	  config.bash_length = 15;
      
      if(strstr(line, "ASKDIR"))
	config.flags |= CONFIG_ASKDIR;

      if(strstr(line, "LABELS"))
	if(strstr(line, "ON"))
	  config.flags |= CONFIG_SHOWLABELS;
	else if(strstr(line, "OFF"))
	  config.flags &= ~CONFIG_SHOWLABELS;
      
      if(strstr(line, "FOCUS")) {
	if(strstr(line, "COMMAND"))
	  config.flags |= CONFIG_FOCUS_COMMAND;
	if(strstr(line, "DIRECTORY"))
	  config.flags |= CONFIG_FOCUS_DIRECTORY;
	if(strstr(line, "HISTORY"))
	  config.flags |= CONFIG_FOCUS_HISTORY;
      }

      if(strstr(line, "SIZE"))
	if(sscanf(line, "SIZE %i %i", &config.x, &config.y) < 2)
	  config.x = config.y = -1;

      if(strstr(line, "POSITION"))
	if(sscanf(line, "POSITION %i %i", &config.x_pos, &config.y_pos) < 2)
	  config.x_pos = config.y_pos = -1;

      if(strstr(line, "PERSIST"))
	if(strstr(line, "ON"))
	  config.flags |= CONFIG_PERSIST;
	else
	  config.flags &= ~CONFIG_PERSIST;

      if(strstr(line, "[HISTORY]")) {
	state = 1;
	config.history = calloc(config.config_length + config.bash_length, 
				sizeof(char *));
	i = 0;
      }
    }
    else if (state == 1 && i < config.config_length) {
      config.history[i] = calloc(strlen(line) + 1, sizeof(char));
      strcpy(config.history[i], line);
      /* Get wrid of unnecessary endline characters */
      chr = strrchr(config.history[i], '\n');
      if(chr)
	*chr = '\0';
      i++;
    }
    
  }
  
  /* Store the actual number of commands in the history */
  config.config_number = i;

  /* Close the file */
  fclose(stream);


  if(config.flags & CONFIG_BASH_HISTORY) {
    /* Get the bash_history's complete file name */
    home_dir = calloc((strlen(home_env) + 2 + strlen(".bash_history")), 1);
    strcpy(home_dir, home_env);
    strcat(home_dir, "/");
    strcat(home_dir, ".bash_history");

    /* Open the file */
    stream = fopen(home_dir, "r");
    free(home_dir);

    for(j = i; j < config.bash_length + i; j++) {
      fgets(line, 1024, stream);
      if(is_comment(line)) {
	j--;
	continue;
      }
      config.history[j] = calloc(strlen(line) + 1, sizeof(char));
      strcpy(config.history[j], line);
      /* Get wrid of unnecessary endline characters */
      chr = strrchr(config.history[j], '\n');
      if(chr)
	*chr = '\0';
    }

    config.bash_number = j - i;

    fclose(stream);
  }

  
  return TRUE;
}

int read_bash_history()
{
  /* This probably isn't going to be the implementation */  
  return TRUE;
}

GList *create_history_list()
{
  GList *list=NULL;
  int i = 0, j = 0;

  /* create the list */
  /*   list = g_list_alloc(); */

  /* add the elements in order */
  if(config.flags & CONFIG_BASH_FIRST) {
    /*    printf("Using Bash first.\n"); */
    if(config.flags & CONFIG_BASH_HISTORY)
      for(i = config.config_number; 
	  i < config.bash_number + config.bash_number; i++)
	if(config.history[i])
	  list = g_list_append(list, (gpointer)config.history[i]);
    if(config.flags & CONFIG_CONFIG_HISTORY)
      for(j=0; j < config.config_number; j++)
	if(config.history[j])
	  list = g_list_append(list, (gpointer)config.history[j]);
  }
  else {
    /*    printf("Using Config First.\n"); */
    if(config.flags & CONFIG_CONFIG_HISTORY)
      for(i=0; i < config.config_number; i++)
	if(config.history[i])
	  list = g_list_append(list, (gpointer)config.history[i]);
    if(config.flags & CONFIG_BASH_HISTORY)
      for(j = i; j < config.bash_number + i; j++)
	if(config.history[j])
	  list = g_list_append(list, (gpointer)config.history[j]);
  }
#ifdef DEBUG
  if(config.flags & CONFIG_CONFIG_HISTORY)
    printf("Using Config History. (%i commands in history of %i, %i possible)\n", i, config.config_number, config.config_length);
  if(config.flags & CONFIG_BASH_HISTORY)
    printf("Using Bash History.\n");
  printf("The list is %i elements long.\n", g_list_length(list)); 
  if(config.flags & CONFIG_BASH_FIRST)
    printf("Bash is first.\n");
  else
    printf("Config is first.\n");
#endif
    return list;
}

void strip_comment(char *string)
{
  char *place;
  int i = 0, j = 0, size;

  size = strlen(string);
  /* Get wrid of the comments */
  place = string;
  while((place = strchr(place, '#'))){
    if(string[(place - string) -1] != '\\')
      *place = '\0';
    place++;
  }

  /* Replace the escape sequences */
  place = calloc(1, strlen(string));
  while(string[i] != '\0'){
    if(string[i] == '\\')
      place[j] = string[++i];
    else
      place[j] = string[i];

    i++;
    j++;
  }

  EAZERO(string, size);
  strncpy(string, place, size);
  free(place);
  return;
}

int is_comment(char *string)
{
  int i=0;

  while (string[i] != '\0'){
    if (string[i] == '#')
      return 1;
    if (string[i] != ' ' && string[i] != '\t')
      return 0;
    i++;
  }
  return 0;
}
