/******************************************************************************
 * xrun - a quick and easy way to run programs under X without having to drop
 * to a terminal window.
 Copyright (C) 1998  by Christopher Lansdown

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/

#include "main.h"
#ifdef WITH_ICON
#include "xrun.xpm"
#endif

/*****************************************************************************/
/* Begin Global Variables */

GtkWidget *window;
GtkWidget *mainbox;
GtkWidget *topbox; 
GtkWidget *bottombox;
GtkWidget *dir;
GtkWidget *input;
GtkWidget *run;
GtkWidget *browse;
GtkWidget *find;
Config config;

/* End Global Variables */
/*****************************************************************************/ 

int main(int argc, char **argv)
{
  char name[128];
  GList *list;
  GtkWidget *box;
  GtkWidget *label;
  int i;

#ifdef WITH_ICON
  GdkPixmap *icon;
  GdkBitmap *mask;
  GtkStyle *style;
#endif

  /* start up gtk */
  gtk_init(&argc, &argv);

  /* Initialize everything */
  if(!config_init()) {
    printf("Error: unable to initialize.\n");
    return 1;
  }

  /* Prepare the name */
  sprintf(name, "Xrun v%s", VERSION);

  /* Set up the window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_widget_set_usize(GTK_WIDGET(window), config.x, config.y);
  gtk_widget_set_uposition(GTK_WIDGET(window), config.x_pos, config.y_pos);

  gtk_window_set_title(GTK_WINDOW(window), name);
  gtk_signal_connect(GTK_OBJECT (window), "delete_event", 
		     (GtkSignalFunc) gtk_exit, NULL);
  signal(SIGINT, (void *)gtk_main_quit);

  /* Set up the boxes */
  mainbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(mainbox);
  gtk_container_add(GTK_CONTAINER(window), mainbox);

  /* Set up the text entry boxes */
  /* The directory */
  if(config.flags & CONFIG_ASKDIR) {
    topbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(topbox);
    
    if(config.flags & CONFIG_SHOWLABELS) {
      label = gtk_label_new("Directory:  ");
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX (topbox), label, FALSE, FALSE, 0);
    }

    dir = gtk_entry_new();
    gtk_widget_show(dir);
    gtk_box_pack_start(GTK_BOX (topbox), dir, TRUE, TRUE, 0);
    
    gtk_box_pack_start(GTK_BOX(mainbox), topbox, TRUE, TRUE, 0);
  }
  /* The command */
  topbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(topbox);

  if(config.flags & CONFIG_SHOWLABELS) {
    label = gtk_label_new("Command:");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX (topbox), label, FALSE, FALSE, 0);
  }

  input = gtk_combo_new();
  gtk_widget_show(input);
  list = create_history_list();
  if(list)
    gtk_combo_set_popdown_strings(GTK_COMBO (input), list);
  gtk_entry_set_text(GTK_ENTRY (GTK_COMBO (input)->entry), 
		     "");
  gtk_box_pack_start(GTK_BOX(topbox), input, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT (GTK_COMBO(input)->entry), "activate", 
		     GTK_SIGNAL_FUNC (input_callback), input);
  gtk_box_pack_start(GTK_BOX(mainbox), topbox, TRUE, TRUE, 0);

  /* Set up the buttons */
  bottombox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(bottombox);

  /* The run button */
  run = gtk_button_new_with_label("Run");
  gtk_widget_show(run);
  gtk_box_pack_start(GTK_BOX(bottombox), run, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (run), "clicked",
		      GTK_SIGNAL_FUNC (input_callback), input);


  /* The browse Button */
  browse = gtk_button_new_with_label("Browse");
  gtk_widget_show(browse);
  gtk_box_pack_start(GTK_BOX(bottombox), browse, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (browse), "clicked",
		      GTK_SIGNAL_FUNC (browse_callback), input);


  gtk_box_pack_start(GTK_BOX(mainbox), bottombox, TRUE, TRUE, 0);

  /* show the window */
  gtk_widget_show(window);


  /* Clear the preceding events.  This isn't perfect, but it works so far */
  for(i = 0; i < 16; i++)
    gtk_main_iteration();

  /* Get the focus right */
  if(config.flags & CONFIG_FOCUS_COMMAND)
    gtk_widget_grab_focus(GTK_COMBO(input)->entry);
  if(config.flags & CONFIG_FOCUS_DIRECTORY && config.flags & CONFIG_ASKDIR)
    gtk_widget_grab_focus(dir);
  if(config.flags & CONFIG_FOCUS_HISTORY)
    gtk_signal_emit_by_name(GTK_OBJECT(GTK_COMBO(input)->button), "clicked");

#ifdef WITH_ICON
  /* Set up the icon */
  style = gtk_widget_get_style(window);
  icon = gdk_pixmap_create_from_xpm_d(window->window, 
				      &mask, 
				      &style->bg[GTK_STATE_NORMAL], 
				      (gchar **)xrun_xpm);
  gdk_window_set_icon(window->window, NULL, icon, mask);
#endif
    
  /* Go into the main event loop */
  gtk_main();

  return 0;
}


void input_callback(GtkWidget *widget, GtkWidget *entry_box)
{
  gchar *entry_text;
  char *buffer, *tmp;
  struct stat status;
  int fd, tlen = 0, len=0, finished = 0, history=0;
  
  /* Change to the Specified Directory */
  if(config.flags & CONFIG_ASKDIR)
    chdir(gtk_entry_get_text(GTK_ENTRY (dir)));

  entry_text = gtk_entry_get_text(GTK_ENTRY (GTK_COMBO (entry_box)->entry));

    /* Store the command in the history */
    if(config.flags & CONFIG_HISTORY_EDITABLE) {
      stat(config.config, &status);
      buffer = malloc(status.st_size + 1);
      if(buffer) {
	buffer[status.st_size] = '\0';

	/* Read the config file into memory */
	fd = open(config.config, O_RDONLY);
	read(fd, buffer, status.st_size);
	close(fd);
	
	/* open it up for writing */
	fd = open(config.config, O_WRONLY | O_TRUNC);
	
	tmp = buffer;
	while(!finished) {
	  tmp = getline(&buffer[tlen]);
	  if(!tmp) {
	    finished = 1;
	    continue;
	  }
	  
	  /* Keep track of how many commands we have in the the history */
	  if(history)
	    history++;
	  
	  /* Figure out if we are in the history */
	  if(strstr(tmp, "[History]"))
	    history=1;
	  
	  /* Add the tlength to the total length */
	  len = strlen(tmp);
	  tlen += len;
	  /* Take the newline character into account */
	  tlen++; 

	  /* Print the current line if it hasn't been already */
	  if(!is_same(tmp, entry_text) && history <= config.config_length) {
	    write(fd, tmp, len);
	    write(fd, "\n", 1);
	  }
	  /* Once the [History] tag is printed, include the current tag */
	  if(history == 1) {
	    write(fd, entry_text, strlen(entry_text));
	    write(fd, "\n", 1);
	  }
	  
	  
	  
	}
      }
      else
	printf("Unable to allocate memory for History write.\n");
    }
    
    if(config.flags & CONFIG_PERSIST)
      if(fork())
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(input)->entry), "");
      else
	execute(entry_text);
    else
      execute(entry_text);
  return;
  
}

char *getline(char *text)
{
  static char *line = NULL;
  int i = 0;
  
  while(text[i] != '\n' && text[i] != '\0')
    i++;

  if(line) {
    free(line);
    line = NULL;
  }

  if(text[0] != '\0') {
    line = calloc(i+1, sizeof(char));
    strncpy(line, text, i);
    line[i] = '\0';
  }
  else
    line = NULL;

  return line;
}

void browse_callback(GtkWidget *widget, GtkWidget *data)
{
  find = gtk_file_selection_new("Find program");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (find)->ok_button),
                            "clicked", (GtkSignalFunc) find_ok, find );
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (find)->cancel_button),
                            "clicked", (GtkSignalFunc) find_cancel, find );
  gtk_widget_show(find);
  return;
}

void find_ok(GtkWidget *widget, GtkWidget *filew)
{
  gtk_entry_append_text(GTK_ENTRY(GTK_COMBO(input)->entry),
			gtk_file_selection_get_filename(GTK_FILE_SELECTION (filew)));
  gtk_widget_hide(filew);
  gtk_widget_destroy(filew);
  return;
}

void find_cancel(GtkWidget *widget, GtkWidget *filew)
{
  gtk_widget_hide(filew);
  gtk_widget_destroy(filew);
  return;
}

void execute(char *command)
{

  execl("/bin/sh", "sh", "-c", command, NULL);

  return;
}



